from typing import List

from attr import attrs, attrib
from pychord import Chord


@attrs
class Time:
    measure = attrib(type=int)
    beat = attrib(type=int, default=0)
    tick = attrib(type=int, default=0)

    def __str__(self):
        return f"{self.measure}:{self.beat}:{self.tick}"

    def as_seconds(self, tempo, timebase, beat):
        return (self.tick / timebase + self.beat + self.measure * beat) / tempo * 60


class ChordEvent:
    def __init__(self, time_: Time, duration: Time, chord: Chord):
        self.time = time_
        self.duration = duration
        self.chord = chord


class ChordTrack:
    def __init__(self):
        self.chords: List[ChordEvent] = []

    def append(self, chord: ChordEvent):
        self.chords.append(chord)


class Song:
    def __init__(self, tempo=120, timebase=240, beat=4, chord_track=None):
        if chord_track is None:
            chord_track = ChordTrack()
        self.tempo = tempo
        self.timebase = timebase
        self.beat = beat
        self.chord_track: ChordTrack = chord_track



from pychord import Chord, ChordProgression

from pypop.song import ChordTrack, ChordEvent, Time, Song


def compose():
    """ Compose a pop song

    This is currently a mock-up code.
    """
    chords = [Chord("C"), Chord("F"), Chord("G"), Chord("C")]
    chord_track = ChordTrack()
    for i, chord in enumerate(chords):
        event = ChordEvent(time_=Time(measure=i + 1), duration=Time(measure=1), chord=chord)
        chord_track.append(event)
    song = Song(chord_track=chord_track)
    return song



from setuptools import setup, find_packages
version = '0.1.1'

setup(
    name='pypop',
    version=version,
    description="A python library to compose pop songs automatically.",
    keywords='music pops',
    author='Yuma Mihira',
    url='https://github.com/yuma-m/pypop',
    license='MIT',
    python_requires='>=3.6',
    install_requires=[
        'attrs',
        'pychord',
    ],
    packages=find_packages(exclude=['test']),
    long_description=open('README.rst').read(),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Multimedia :: Sound/Audio",
        "License :: OSI Approved :: MIT License",
    ],
)



import time

from pypop.song import Song


class Player:
    def __init__(self):
        pass

    def play(self, song: Song):
        for event in song.chord_track.chords:
            print(f"{event.time} {event.chord}")
            time.sleep(event.duration.as_seconds(tempo=song.tempo, timebase=song.timebase, beat=song.beat))
